#pragma once

#include <list>
#include <map>

#include "flexpage.h"
#include "memory.h"
#include "page_owner.h"



/* Collection types. */

typedef struct { Flexpage *flexpage; PageOwner *owner; } QueueEntry;
typedef std::list<QueueEntry> Queue;

typedef std::pair<Flexpage *, Queue::iterator> Position;
typedef std::map<Flexpage *, Queue::iterator> Positions;



/* A queue of managed pages. */

class PageQueue
{
protected:

    /* Helper methods. */

    virtual void discard(Queue &queue, Memory *memory);

    virtual bool remove(Queue &queue, Positions &positions, PageOwner *owner, Flexpage *flexpage);

public:
    virtual ~PageQueue()
    {
    }

    virtual void close(Memory *memory) = 0;

    virtual void pop(PageOwner **owner, Flexpage **flexpage) = 0;

    virtual void push(PageOwner *owner, Flexpage *flexpage) = 0;

    virtual void push_front(PageOwner *owner, Flexpage *flexpage) = 0;

    virtual bool remove(PageOwner *owner, Flexpage *flexpage) = 0;
};

// vim: tabstop=4 expandtab shiftwidth=4
